﻿#include <string>
#include <sstream>
#include <iostream>
#include <fstream>

#include "utf8.h"

#include <vector>

using namespace std;

#ifdef _WIN64
#define OS 1
#elif _WIN32
#define OS 1
#elif __APPLE__
#define OS 2
#elif __linux
#define OS 3
#elif __unix 

#elif __posix

#endif

// Fix line break for Windows ( "\r" + "\n" )
void FixLineBreakWin(wstring& data)
{
	int x = 0;
	while (1)
	{
		if (data[x] == (wchar_t)10)
		{
			if (x != 0)
			{
				if (data[x - 1] != (wchar_t)13)
				{
					data.insert(x, 1, (wchar_t)13);
				}
			}
			else
			{
				data.insert(x, 1, (wchar_t)13);
			}
			x++;
		}
		x++;
		if (x == data.length()) break;
	}
}

// Fix line break for Apple ( from "\n" to "\r" )
void FixLineBreakMac(wstring& data)
{
	int x = 0;
	while (1)		
	{
		if (data[x] == (wchar_t)10)
		{
			data.erase(x, 1);
			data.insert(x, 1, (wchar_t)13);
		}
		x++;
		if (x == data.length()) break;
	}
}

bool valid_utf8_file(const char* file_name)
{
	ifstream ifs(file_name);
	if (!ifs)
		return false; // even better, throw here

	istreambuf_iterator<char> it(ifs.rdbuf());
	istreambuf_iterator<char> eos;

	return utf8::is_valid(it, eos);
}

// only UTF16LE and UTF8
std::wstring readFileSync(const char* filename, const char* options)
{
	string opt = options;
	short type = 0;
	wstring str_NULL;
	string st = " ";
	str_NULL.assign(st.begin(), st.end());


	short z = (short)opt.find("utf8");
	if (z > -1)
	{
		type = 1;  // type == 1  = utf8
	}

	z = (short)opt.find("utf16le");
	if (z > -1) 
	{
		type = 2;  // type == 2  = utf16le
	}

	if (type == 1)
	{
		std::ifstream wif(filename);
		if (!wif)  return str_NULL;

		istreambuf_iterator<char> it(wif.rdbuf());
		istreambuf_iterator<char> eos;
		if (utf8::is_valid(it, eos) == 0) return str_NULL;


		std::ifstream rtext(filename);
		istreambuf_iterator<char> it_f(rtext.rdbuf());
		istreambuf_iterator<char> eos_f;

		std::wstring wstr;
		utf8::utf8to16(it_f, eos_f, back_inserter(wstr));
		wstr.erase(0, 1);

		if (OS == 1) FixLineBreakWin(wstr);
		if (OS == 2) FixLineBreakMac(wstr);
		return wstr;

	}
	else if (type == 2)
	{
		std::ifstream wif(filename);
		std::stringstream wss;
		wss << wif.rdbuf();
		std::string  const &str = wss.str();
		std::wstring wstr;
		wstr.resize(str.size() / sizeof(wchar_t));
		std::memcpy(&wstr[0], str.c_str(), str.size()); // copy data into wstring
		wstr.erase(0, 1);
		return wstr;
	}
	else
	{
		return str_NULL;
	}

}

// only UTF16LE and UTF8
void writeFileSync(const char* myFile, wstring& ws, const char* options)
{
	string opt = options;
	int z = (int)opt.find("utf16le");
	if (z > -1)
	{
		std::ofstream outFile(myFile, std::ios::out | std::ios::binary);
		if ((int)ws[0] != 65279)
		{  	
	    	const char ch = 0xFF;
		    outFile.write(&ch, sizeof(char));
		    const char cf = 0xFE;
			outFile.write(&cf, sizeof(char));
	    }

		outFile.write((char *)ws.c_str(), ws.length() * sizeof(wchar_t));
		outFile.close();
		return;
	}

	z = (int)opt.find("utf8");
	if (z > -1)
	{
		vector <unsigned char> utf8result;
		utf8::utf16to8(ws.begin(), ws.end(), back_inserter(utf8result));
		unsigned char* buff = &utf8result[0];
		

		std::ofstream outFile(myFile, std::ios::out | std::ios::binary);

		const char ch = 0xEF;
		outFile.write(&ch, sizeof(char));
		const char cf = 0xBB;
		outFile.write(&cf, sizeof(char));
		const char ct = 0xBF;
		outFile.write(&ct, sizeof(char));
		
		outFile.write((const char*)buff, utf8result.size());
		
		utf8result.clear();
		utf8result.shrink_to_fit();

		outFile.close();
	}
}



int main()
{
	wchar_t p13 = (wchar_t)13;
	wchar_t p10 = (wchar_t)10;

	std::wstringstream wss;
	wss << p13 << p10;

	wstring newLine = wss.str();


	wstring sample;
	sample.append(newLine); 
	sample.append(L"おはよう 林檎 Apple Intel");
	sample.append(newLine);
	sample.append(newLine);
	sample.append(newLine);
	sample.append(L"おはよう 林檎 Samsung");


	writeFileSync("a_utf16.txt", sample, "utf16le");

	wstring strw;
	strw = readFileSync("a_utf16.txt", "utf16le");

	writeFileSync("b_utf16.txt", strw, "utf16le");

	writeFileSync("c_utf8.txt", strw, "utf8");


	wstring strw2;
	strw2 = readFileSync("c_utf8.txt", "utf8");


	writeFileSync("d_utf16.txt", strw2, "utf16le");
	writeFileSync("e_utf8.txt", strw2, "utf8");


	system("pause");
	return 0;
}